'''
@author: Telecoda - entry for Pyweek 12
'''

import os

from Tiles import *

from Enemy import *

BOARD_WIDTH = 23
BOARD_HEIGHT = 23

BLANK_TILE_IMAGE_ID = 246

class GameBoard(object):
    '''
    Main controlling class for the Nine Times game board
    '''

    def __init__(self, screen, images,main_engine):
        '''
        Constructor
        '''
        self.screen = screen
 
        self.images = images
        
        # reference to parent class
        self.main_engine = main_engine
        
        self.visible = True
        
        self.enemies = [] # an empty list to contain enemies

        self.board_tiles = {}
        for x in range(BOARD_WIDTH):
            for y in range(BOARD_HEIGHT):
                blank_tile = BlankTile(image=self.images[4])
                # set position of tile on board
                blank_tile.set_board_pos(x,y)
                self.board_tiles[(x,y)]= blank_tile
    
    def animation_tick(self):
        """ call animation tick method of all object on the game board """
        # animate player
        self.player.animation_tick()
        # animate enemies
        for enemy in self.enemies:
            enemy.animation_tick()
        # animate board tiles
        for x in range (0,BOARD_WIDTH):
            for y in range (0,BOARD_HEIGHT):
                tile = self.board_tiles[(x,y)]
                if(tile is not None):
                    tile.animation_tick()
    
    def check_collisions(self):
        # check if player has collided with any of the enemies
        for enemy in self.enemies:
            #
            if(self.player.board_x == enemy.board_x and self.player.board_y == enemy.board_y):
                # boom!
                self.main_engine.player_dies()
                # remove enemy from board
                self.enemies.remove(enemy)
    def draw (self):
        '''
        draws game board
        '''
        if(not self.visible):
            return
        
        # draw tiles on board
        for x in range(0,BOARD_WIDTH):
            for y in range(0,BOARD_HEIGHT):
                # draw tile
                tile = self.board_tiles[(x,y)]
                tile.draw(self.screen)
        
        # draw player on board
        self.player.draw(self.screen)
    
        # draw enemies on board
        for enemy in self.enemies:
            enemy.draw(self.screen)
        
    def load_level(self, level):
        ''' load tile details from level file '''
        # open a file and covert it to game tiles 
        filename = os.path.join('data','levels','level'+str(level)+".txt")
        #  
        f = open(filename, "rb")
        level_data = f.read().split('\n')
        self.board_tiles = {}
        self.counter_tile_total=0
        self.enemies=[]
        counter_images = [self.images[247],
                          self.images[299],
                            self.images[298],
                            self.images[297],
                            self.images[296],
                            self.images[295],
                            self.images[294],
                            self.images[293],
                            self.images[292],
                            self.images[291] ]
        for x in range(BOARD_WIDTH):
            for y in range(BOARD_HEIGHT):
                tile_char = level_data[y][x]
                if(tile_char==' '):
                    # blank tile
                    tile = BlankTile(image=self.images[BLANK_TILE_IMAGE_ID])
                elif(tile_char=='x'):
                    # blank tile
                    tile = BlankTile(image=self.images[BLANK_TILE_IMAGE_ID])
                elif(tile_char=='S'):
                    # start point for player
                    self.startx=x
                    self.starty=y
                    self.player.set_board_pos(self.startx,self.starty)
                    tile = BlankTile(image=self.images[BLANK_TILE_IMAGE_ID])
                elif(tile_char=='E'):
                    # start point for enemy
                    enemy_x=x
                    enemy_y=y
                    enemy = Enemy()
                    enemy.set_image(self.images[268+10])

                    enemy.set_board_pos(enemy_x,enemy_y)
                    self.enemies.append(enemy)
                    tile = BlankTile(image=self.images[BLANK_TILE_IMAGE_ID])
                elif(tile_char=='A'):
                    # solid tile - left side
                    tile = SolidTile(image=self.images[13])
                elif(tile_char=='a'):
                    # solid tile - left side brown
                    tile = SolidTile(image=self.images[13+12])
                elif(tile_char=='Q'):
                    # solid tile - top left
                    tile = SolidTile(image=self.images[89])
                elif(tile_char=='q'):
                    # solid tile - top left brown
                    tile = SolidTile(image=self.images[89+12])
                elif(tile_char=='P'):
                    # solid tile - top right
                    tile = SolidTile(image=self.images[127])
                elif(tile_char=='p'):
                    # solid tile - top right brown
                    tile = SolidTile(image=self.images[127+12])
                elif(tile_char=='L'):
                    # solid tile - right side
                    tile = SolidTile(image=self.images[13])
                elif(tile_char=='l'):
                    # solid tile - right side brown
                    tile = SolidTile(image=self.images[13+12])
                elif(tile_char=='Z'):
                    # solid tile - bottom left
                    tile = SolidTile(image=self.images[14])
                elif(tile_char=='z'):
                    # solid tile - bottom left brown
                    tile = SolidTile(image=self.images[14+12])
                elif(tile_char=='M'):
                    # solid tile - bottom right
                    tile = SolidTile(image=self.images[128])
                elif(tile_char=='m'):
                    # solid tile - bottom right brown
                    tile = SolidTile(image=self.images[128+12])
                elif(tile_char=='B'):
                    # solid tile - bottom row
                    tile = SolidTile(image=self.images[52])
                elif(tile_char=='b'):
                    # solid tile - bottom row brown
                    tile = SolidTile(image=self.images[52+12])
                elif(tile_char=='T'):
                    # solid tile - top row
                    tile = SolidTile(image=self.images[52])
                elif(tile_char=='t'):
                    # solid tile - top row brown
                    tile = SolidTile(image=self.images[52+12])
                elif(tile_char=='1'):
                    # counter tile 1
                    tile = CounterTile(counter_images,1)
                    self.counter_tile_total+=1
                elif(tile_char=='2'):
                    # counter tile 2
                    tile = CounterTile(counter_images,2)
                    self.counter_tile_total+=1
                elif(tile_char=='3'):
                    # counter tile 3
                    tile = CounterTile(counter_images,3)
                    self.counter_tile_total+=1
                elif(tile_char=='4'):
                    # counter tile 4
                    tile = CounterTile(counter_images,4)
                    self.counter_tile_total+=1
                elif(tile_char=='5'):
                    # counter tile 5
                    tile = CounterTile(counter_images,5)
                    self.counter_tile_total+=1
                elif(tile_char=='6'):
                    # counter tile 6
                    tile = CounterTile(counter_images,6)
                    self.counter_tile_total+=1
                elif(tile_char=='7'):
                    # counter tile 7
                    tile = CounterTile(counter_images,7)
                    self.counter_tile_total+=1
                elif(tile_char=='8'):
                    # counter tile 8
                    tile = CounterTile(counter_images,8)
                    self.counter_tile_total+=1
                elif(tile_char=='9'):
                    # counter tile 9
                    tile = CounterTile(counter_images,9)
                    self.counter_tile_total+=1
                else:
                    print str(x)+','+str(y)+':'+tile_char + ' - unexpected char.'
                    continue
                # set position of tile on board
                tile.set_board_pos(x,y)
                self.board_tiles[(x,y)]= tile
        # after tile data are other level variables
        # feels a bit hacky but works if data perfect
        
        self.level_vars = {}
        row = BOARD_HEIGHT
        current_line = level_data[row]
        for current_line in level_data[row:]:
            key,value = current_line.split(':')
            self.level_vars[key] = value
        

        
    def move_player_down(self,player):
        # check if live player or enemy can move
        # check for edge of board
        if(player.board_y == BOARD_HEIGHT-1):
            return # at bottom edge of board
        # get tile from next location on grid
        next_x = player.board_x
        next_y = player.board_y+1
        next_tile = self.board_tiles[(next_x,next_y)]
        # if tile is solid player cannot move into it
        if(next_tile.isSolid):
            # TODO play bump sound
            return
        else:
            # move down
            player.set_board_pos(next_x,next_y)
            # if next tile is a counter
            if (type(next_tile) == CounterTile):
                # decrease counter
                next_tile.decrease_count()
                if(next_tile.remaining_count==0):
                    self.tile_completed(next_tile)
            # check for collisions with enemies
            self.check_collisions()

    def move_player_left(self,player):
        # check if live player or enemy can move
        # check for edge of board
        if(player.board_x == 0):
            return # at left edge of board
        # get tile from next location on grid
        next_x = player.board_x-1
        next_y = player.board_y
        next_tile = self.board_tiles[(next_x,next_y)]
        # if tile is solid player cannot move into it
        if(next_tile.isSolid):
            # TODO play bump sound
            return
        else:
            # move left
            player.set_board_pos(next_x,next_y)
            # if next tile is a counter
            if (type(next_tile) == CounterTile):
                # decrease counter
                next_tile.decrease_count()
                if(next_tile.remaining_count==0):
                    self.tile_completed(next_tile)
            # check for collisions with enemies
            self.check_collisions()

    def move_player_right(self,player):
        # check if live player or enemy can move
        # check for edge of board
        if(player.board_x == BOARD_WIDTH-1):
            return # at right edge of board
        # get tile from next location on grid
        next_x = player.board_x+1
        next_y = player.board_y
        next_tile = self.board_tiles[(next_x,next_y)]
        # if tile is solid player cannot move into it
        if(next_tile.isSolid):
            # TODO play bump sound
            return
        else:
            # move right
            player.set_board_pos(next_x,next_y)
            # if next tile is a counter
            if (type(next_tile) == CounterTile):
                # increase counter
                next_tile.increase_count()
            # check for collisions with enemies
            self.check_collisions()
    
    def move_player_up(self,player):
        # check if live player or enemy can move
        # check for edge of board
        if(player.board_y == 0):
            return # at top edge of board
        # get tile from next location on grid
        next_x = player.board_x
        next_y = player.board_y-1
        next_tile = self.board_tiles[(next_x,next_y)]
        # if tile is solid player cannot move into it
        if(next_tile.isSolid):
            # TODO play bump sound
            return
        else:
            # move up
            player.set_board_pos(next_x,next_y)
            # if next tile is a counter
            if (type(next_tile) == CounterTile):
                # increase counter
                next_tile.increase_count()
            # check for collisions with enemies
            self.check_collisions()

    def set_player(self, player):
        '''
        '''
        self.player = player
        
    def tile_completed(self,current_tile):
        x,y = current_tile.x, current_tile.y
        #
        blank_tile = BlankTile(image=self.images[BLANK_TILE_IMAGE_ID])
        blank_tile.set_board_pos(x,y)
        # add bag into board
        self.board_tiles[x,y]=blank_tile
        
        # decrease tile counter
        self.counter_tile_total -=1
        
        if(self.counter_tile_total==0):
            self.main_engine.init_end_level()
            


    def add_enemy(self, enemy):
        '''
        '''
        self.enemies.append(enemy)

