'''
@author: Telecoda
'''
from GameObject import *

ANIMATION_LOOP = 1
ANIMATION_RUNONCE =2
TILE_SIZE_PIXELS = 32

class Player(GameObject):
    '''
    Player object
    '''


    def __init__(self):
        '''
        Constructor
        '''
        self.isDestructible = False
        self.isSolid = False
        self.isExploding = False
 
        self.visible = True
        self.animStartFrame=0
        self.animEndFrame=0
        self.animCurrentFrame=0
        self.animFrameTick=0
        self.animCurrentTick=0
        self.animRunning=False
        self.animStyle=ANIMATION_LOOP
        self.board_x=0
        self.board_y=0
        self.screen_x=0
        self.screen_y=0

        self.images = {}

    def animation_tick(self):
        if(self.animRunning):
            # increment tick counter
            self.animCurrentTick += 1
            if(self.animCurrentTick == self.animFrameTick):
                # if number of ticks reached increment frame counter
                self.animCurrentTick = 0
                # increment frame counter
                self.animCurrentFrame += 1
                # check if end

                
                if(self.animCurrentFrame == self.animEndFrame):
                    # if run once stop
                    if(self.animStyle == ANIMATION_RUNONCE):
                        self.animRunning = False
                        self.animCurrentFrame=self.animEndFrame
                    else:
                        # start again
                        self.animCurrentFrame=self.animStartFrame
                    # do any action for end of animation here
                    self.executeEndOfAnimationMethod()

        
    def draw (self,screen):
        '''
        draws game object on current screen
        '''
        if(not self.visible):
            return
        screen.blit(self.images[self.animCurrentFrame], [self.screen_x,self.screen_y], None,0)

        
    def execute_end_of_anim_meth(self):
        '''
        dynamically call a method at the end of an animation
        '''
        if(self.end_of_anim_meth == None):
            return
        self.self.end_of_anim_meth(self)
        

    def set_image(self, image):
        self.images[0] = image
    def set_images(self, images):
        self.images = images

    def set_board_pos(self,x,y):
        # convert x,y pos to screen x,y pos
        self.screen_x = x * TILE_SIZE_PIXELS
        self.screen_y = y * TILE_SIZE_PIXELS
        
        self.board_x = x
        self.board_y = y


    def set_end_of_anim_meth(self, method):
        '''
        set method to call at end of animation
        '''
        self.self.end_of_anim_meth = method



