'''
@author: Telecoda - entry for Pyweek 12
'''

from GameObject import *
from pygame.locals import *

TILE_SIZE_PIXELS = 32

class Tile(GameObject):
    '''
    Tile class used as object on game board
    '''


    def __init__(self,image):
        '''
        Constructor
        '''
        GameObject.__init__(self)
        self.image = image
        self.sx = 0
        self.sy = 0

    def draw(self, screen):

        screen.blit(self.image, [self.sx,self.sy], None,0)
        
    def set_board_pos(self,x,y):
        # convert x,y pos to screen x,y pos
        self.sx = x * TILE_SIZE_PIXELS
        self.sy = y * TILE_SIZE_PIXELS
        
        self.x = x
        self.y = y

            
class BlankTile(Tile):
    '''
    Blank tile on the board 
    '''
    def __init__(self,image):
        '''
        Constructor
        '''
        Tile.__init__(self,image)
        self.isSolid = False

class SolidTile(Tile):
    '''
    Solid tile on the board - player cannot enter solid tiles 
    '''
    def __init__(self,image):
        '''
        Constructor
        '''
        Tile.__init__(self,image)
        self.isSolid = True

class CounterTile(Tile):
    '''
    Counter tile on the board 
    '''
    def __init__(self,images,count):
        '''
        Constructor
        '''
        Tile.__init__(self,images[count])
        self.isSolid = False
        self.images = images
        self.remaining_count = count
        # set image to mat

    def decrease_count(self):
        self.remaining_count -= 1
        if self.remaining_count<0:
            self.remaining_count = 0
        self.image = self.images[self.remaining_count]

    def increase_count(self):
        self.remaining_count += 1
        if self.remaining_count >=9:
            self.remaining_count=9
        self.image = self.images[self.remaining_count]
        
