import sys, pygame, os
from pygame.locals import *

from GameBoard import *
from Player import *

# game constants
SCREEN_WIDTH = 1024
SCREEN_HEIGHT = 736

SPRITE_SIZE_PIXELS=32

FRAME_RATE = 30

KEY_REPEAT = 3

TOTAL_LEVELS=2

MODE_DISPLAY_MENU = 1
MODE_LEVEL_START = 2
MODE_GAME_PLAYING = 3
MODE_GAME_PAUSED = 4
MODE_LEVEL_END = 5
MODE_GAME_OVER = 6
MODE_HIGH_SCORE =7 # who am I kidding in a week!
MODE_GAME_COMPLETED = 8
MODE_LIFE_LOST = 9





def main():
   """ Bunker - by Telecoda
   """

   game = Bunker()
   
   game.main_loop()




class Bunker:
    """ this is the main game class for the game"""
    def __init__(self):
        '''
        Constructor
        '''
        pygame.mixer.pre_init(22050,-16,2,1024)
        pygame.init()
        
        # init screen before converting images
        self.size = self.width, self.height = SCREEN_WIDTH, SCREEN_HEIGHT
        self.screen = pygame.display.set_mode(self.size)
        self.init_images()
        self.init_sounds()

        self.game_board = GameBoard(screen=self.screen,images = self.images, main_engine=self)
        
        player = Player()
        player.set_image(self.images[268])
        player.set_board_pos(11,11)
        self.game_board.set_player(player)
        
        self.speed = [1, 1]
        self.black = 0, 0, 0
 
        pygame.display.set_caption("Bunker")
        self.clock = pygame.time.Clock()
        
        # init font
        self.font = pygame.font.Font(None, 24)

        self.speed = [2, 2]
        self.black = 0, 0, 0
        
        self.init_main_menu()

    def animation(self):
        '''
        method called every main loop
        '''

        self.game_board.animation_tick()
        for enemy in self.game_board.enemies:
            enemy.animation_tick()

    def init_images(self):
        
        self.images = {}
        self.rects = {}
        self.next_image_id = 0
        self.images[self.next_image_id] = pygame.image.load(os.path.join('data','images', 'ball.png'))
        #self.rects[self.next_image_id] = self.images[self.next_image_id].get_rect()
        self.next_image_id+=1
        
        self.images[self.next_image_id] = pygame.image.load(os.path.join('data','images','sprite01.png'))
        #self.rects[self.next_image_id] = self.images[self.next_image_id]
        self.next_image_id+=1
        
        self.images[self.next_image_id] = pygame.image.load(os.path.join('data','images','marioType_Tileset_PIPES1b.png'))
        #self.rects[self.next_image_id] = self.images[self.next_image_id]
        self.hud_image_id = self.next_image_id
        self.next_image_id+=1

        pipes_sprite_sheet = pygame.image.load(os.path.join('data','images',"marioType_Tileset_PIPES1b.png")).convert()
        self.extract_sprites(pipes_sprite_sheet,colorkey=-1)
        print "last_sprite_id" + str(self.next_image_id)
        
        coco_dark_sprite_sheet = pygame.image.load(os.path.join('data','images',"marioType_Tileset_Coco-dark.png")).convert()
        self.extract_sprites(coco_dark_sprite_sheet,colorkey=-1)
        print "last_sprite_id" + str(self.next_image_id)

        bunker_sprite_sheet = pygame.image.load(os.path.join('data','images',"bunker_tileset.png")).convert()
        self.extract_sprites(bunker_sprite_sheet,colorkey=-1)
        print "last_sprite_id" + str(self.next_image_id)
     

    def init_main_menu(self):
        # set up main loop methods
        self.process_events_method = self.process_menu_events
            
        self.animation_method = self.animation
            
        self.render_method = self.render_menu
        
        self.game_mode = MODE_DISPLAY_MENU

        
    
    def extract_sprites(self,sprite_sheet, colorkey):
                        
        width = sprite_sheet.get_width()
        height = sprite_sheet.get_height()

        # loop through sprite image
        for x in range(0,width,SPRITE_SIZE_PIXELS):
            for y in range(0,height,SPRITE_SIZE_PIXELS):
                rect = Rect(x,y,SPRITE_SIZE_PIXELS,SPRITE_SIZE_PIXELS)
                image = pygame.Surface(rect.size).convert()
                image.blit(sprite_sheet, (0, 0), rect)
                if colorkey is not None:
                    if colorkey is -1:
                        colorkey = image.get_at((0, 0))
                    image.set_colorkey(colorkey, RLEACCEL)

                self.images[self.next_image_id] = image
                self.rects[self.next_image_id] = self.images[self.next_image_id]
                self.next_image_id+=1
    
    def init_sounds(self):
        pass
    
    def main_loop(self):
        # main control loop
        # default to main menu
        self.init_main_menu()
        
        self.game_running = True
        
        self.set_all_keys_up()
        
        while self.game_running:
            
            self.process_events_method()
            
            self.animation_method()
            
            # clear scren
            self.screen.fill(self.black)

            self.render_method()
            
            
            # update the clock - throttle the framerate
            self.clock.tick(FRAME_RATE)
            # output the framerate
            #print self.clock.get_fps()
          
            # double buffer flip
            pygame.display.flip()

    def move_player_down(self):
        self.game_board.move_player_down(self.game_board.player)
        
    def move_player_left(self):
        self.game_board.move_player_left(self.game_board.player)
        
    def move_player_right(self):
        self.game_board.move_player_right(self.game_board.player)
        

    def move_player_up(self):
        self.game_board.move_player_up(self.game_board.player)

    def player_dies(self):
        ''' called when a life is lost '''
        # 
        self.lives -=1
        
        self.set_all_keys_up()
        
        if self.lives>0:
            # lost a life
            self.animation_method = self.animation
            self.render_method = self.render_life_lost
            self.process_events_method = self.process_life_lost_events
            self.game_mode = MODE_LIFE_LOST

        else:
            # game over
            self.animation_method = self.animation
            self.render_method = self.render_game_over
            self.process_events_method = self.process_game_over_events
            self.game_mode = MODE_GAME_OVER
        
        
        
    def process_game_events(self):
        # 
            self.update_time_remaining()

            for event in pygame.event.get():
                if event.type == pygame.QUIT: 
                    self.game_running = False
                elif event.type == KEYDOWN and event.key == K_ESCAPE: 
                    self.game_running = False
                
                elif event.type == KEYDOWN and event.key == K_LEFT:
                    self.set_all_keys_up()
                    # left key pressed
                    self.left_key_pressed = True
                    self.left_key_counter = KEY_REPEAT
                elif event.type == KEYDOWN and event.key == K_RIGHT:
                    self.set_all_keys_up()
                    # right key pressed
                    self.right_key_pressed = True
                    self.right_key_counter = KEY_REPEAT
                elif event.type == KEYDOWN and event.key == K_UP:
                    self.set_all_keys_up()
                    # up key pressed
                    self.up_key_pressed = True
                    self.up_key_counter = KEY_REPEAT
                elif event.type == KEYDOWN and event.key == K_DOWN:
                    self.set_all_keys_up()
                    # down key pressed
                    self.down_key_pressed = True
                    self.down_key_counter = KEY_REPEAT
                # key releases
                elif event.type == KEYUP and event.key == K_LEFT:
                    # left key released
                    self.left_key_pressed = False
                    self.left_key_counter = 0
                elif event.type == KEYUP and event.key == K_RIGHT:
                    # right key released
                    self.right_key_pressed = False
                    self.right_key_counter = 0
                elif event.type == KEYUP and event.key == K_UP:
                    # up key released
                    self.up_key_pressed = False
                    self.up_key_counter = 0
                elif event.type == KEYUP and event.key == K_DOWN:
                    # down key released
                    self.down_key_pressed = False
                    self.down_key_counter = 0
            # if keys are still pressed update counters
            if(self.left_key_pressed):
                self.left_key_counter+=1;
                if(self.left_key_counter > KEY_REPEAT):
                    # process left key press
                    self.move_player_left()
                    # reset counter
                    self.left_key_counter=0
            if(self.right_key_pressed):
                self.right_key_counter+=1;
                if(self.right_key_counter > KEY_REPEAT):
                    # process right key press
                    self.move_player_right()
                    # reset counter
                    self.right_key_counter=0
            if(self.up_key_pressed):
                self.up_key_counter+=1;
                if(self.up_key_counter > KEY_REPEAT):
                    # process up key press
                    self.move_player_up()
                    # reset counter
                    self.up_key_counter=0
            if(self.down_key_pressed):
                self.down_key_counter+=1;
                if(self.down_key_counter > KEY_REPEAT):
                    # process down key press
                    self.move_player_down()
                    # reset counter
                    self.down_key_counter=0

    def process_game_over_events (self):
        # wait for key press to start game
        for event in pygame.event.get():
            if event.type == pygame.QUIT: 
                self.game_running = False
            elif event.type == KEYDOWN and event.key == K_ESCAPE: 
                self.game_running = False
            elif event.type == KEYDOWN and event.key == K_SPACE: 
                self.init_main_menu()


    def process_life_lost_events(self):
        # wait for key press to start game
        for event in pygame.event.get():
            if event.type == pygame.QUIT: 
                self.game_running = False
            elif event.type == KEYDOWN and event.key == K_ESCAPE: 
                self.game_running = False
            elif event.type == KEYDOWN and event.key == K_SPACE: 
                self.continue_level()
        
        
    def process_menu_events(self):
        # wait for key press to start game
        for event in pygame.event.get():
            if event.type == pygame.QUIT: 
                self.game_running = False
            elif event.type == KEYDOWN and event.key == K_ESCAPE: 
                self.game_running = False
            elif event.type == KEYDOWN and event.key == K_SPACE: 
                self.start_game()

    def process_end_level_events(self):
        # wait for key press to start game
        for event in pygame.event.get():
            if event.type == pygame.QUIT: 
                self.game_running = False
            elif event.type == KEYDOWN and event.key == K_ESCAPE: 
                self.game_running = False
            elif event.type == KEYDOWN and event.key == K_SPACE: 
                self.next_level()

    def process_game_completed_events(self):
        # wait for key press to start game
        for event in pygame.event.get():
            if event.type == pygame.QUIT: 
                self.game_running = False
            elif event.type == KEYDOWN and event.key == K_ESCAPE: 
                self.game_running = False
            elif event.type == KEYDOWN and event.key == K_SPACE: 
                self.init_main_menu()

            
    def process_start_level_events(self):
        # wait for key press to start game
        for event in pygame.event.get():
            if event.type == pygame.QUIT: 
                self.game_running = False
            elif event.type == KEYDOWN and event.key == K_ESCAPE: 
                self.game_running = False
            elif event.type == KEYDOWN and event.key == K_SPACE: 
                self.start_level()

    def render_board(self):
        self.game_board.draw()

        
    def render_hud(self):
        
        text = 'Level:'+str(self.level)
        
        self.render_text(text,x=746,y=32,colour=(255,255,255))

        text = 'Level:'+self.game_board.level_vars['name']
        
        self.render_text(text,x=746,y=62,colour=(255,255,255))

        text = 'Score:'+str(self.score)
        
        self.render_text(text,x=746,y=92,colour=(255,255,255))

        text = 'Lives:'+str(self.lives)
        
        self.render_text(text,x=746,y=122,colour=(255,255,255))
    
    def render_text(self, text,x,y,colour=(255,255,255),centre=False):
        # render other text
        colour_text = self.font.render(text,1, colour)
        black_text = self.font.render(text,1, (0,0,0))
        textWidth, textHeight = colour_text.get_size() 
        
        if centre:
            text_surface = self.font.render(text,1, (255,255,255))

            # centre text
            cx,cy = self.size
            cx = cx/2
            textWidth, textHeight = text_surface.get_size() 
            x = cx-textWidth/2

        
        self.screen.blit(black_text,[x+2,y+y])
        self.screen.blit(colour_text,[x,y])

    def render_game(self):
        
        self.render_hud()
            
        self.render_board()

    def render_game_over(self):
        
        text = 'Game over man, game over...'
        
        self.render_text(text,x=400,y=300,colour=(255,000,000),centre=True)

        text = 'Press SPACE to continue'
        
        self.render_text(text,x=400,y=400,colour=(255,000,000),centre=True)


    def render_life_lost(self):

        self.render_game()
        
        text = 'You have lost a life'
        
        self.render_text(text,x=400,y=300,colour=(255,000,000),centre=True)

        text = 'Press SPACE to continue'
        
        self.render_text(text,x=400,y=400,colour=(255,000,000),centre=True)


            


    def render_menu(self):
        
        text = 'Bunker - by Telecoda'
        
        self.render_text(text,x=400,y=400,colour=(255,000,000),centre=True)
            
        text = 'Press SPACE to start'
        
        self.render_text(text,x=400,y=500,colour=(255,000,000),centre=True)

    def render_end_level(self):
        
        text = 'Congratulations you have completed'
        
        self.render_text(text,x=300,y=300,colour=(255,000,000),centre=True)

        text = 'Level:' + self.game_board.level_vars['name']
        
        self.render_text(text,x=400,y=400,colour=(255,000,000),centre=True)
            
        text = 'Press SPACE to continue'
        
        self.render_text(text,x=400,y=500,colour=(255,000,000),centre=True)

    def render_game_completed(self):
        
        
        self.render_text(text="Congratulations you completed the game!",x=300,y=200,colour=(255,000,000),centre=True)

        self.render_text(text="You're still here? It's over",x=300,y=300,colour=(255,000,000),centre=True)

        self.render_text(text="Go home...go.",x=400,y=400,colour=(255,000,000),centre=True)
            
        text = 'Press SPACE to return'
        
        self.render_text(text,x=400,y=500,colour=(255,000,000),centre=True)

    

    
    def render_start_level(self):
        
        text = 'Level:' + self.game_board.level_vars['name']
        
        self.render_text(text,x=400,y=400,colour=(255,000,000),centre=True)
            
        text = 'Press SPACE to start'
        
        self.render_text(text,x=400,y=500,colour=(255,000,000),centre=True)
    
    
    
    
    def set_all_keys_up(self):
        ''' reset all key presses, to prevent player making diagonal moves '''
        # reset key pressed indicators
        self.up_key_pressed = False;
        self.down_key_pressed = False;
        self.left_key_pressed = False;
        self.right_key_pressed = False;
        # reset key pressed counters
        self.up_key_counter = False;
        self.down_key_counter = False;
        self.left_key_counter = False;
        self.right_key_counter = False;

    def start_game(self):
        ''' initialise new game '''
        # set up game board
        self.level=1
        self.lives=3
        self.score=0
        
        self.init_start_level()

    def init_end_level(self):
        ''' move to next level '''

        self.set_all_keys_up()
        self.game_mode = MODE_LEVEL_END
        self.animation_method = self.animation
        self.render_method = self.render_end_level
        self.process_events_method = self.process_end_level_events


        
    def init_start_level(self):
        ''' initialise new level '''

        self.game_mode = MODE_LEVEL_START
        self.animation_method = self.animation
        self.render_method = self.render_start_level
        self.process_events_method = self.process_start_level_events

        ''' initialise new level '''
        # code to load level here...
        self.game_board.load_level(self.level)

    def continue_level(self):
        ''' continue level '''

        self.game_mode = MODE_LEVEL_START
        self.animation_method = self.animation
        self.render_method = self.render_start_level
        self.process_events_method = self.process_start_level_events



    def next_level(self):
        # move to next level
        self.level += 1
        if(self.level > TOTAL_LEVELS):
            # end of game!
            self.game_mode = MODE_GAME_COMPLETED
            self.animation_method = self.animation
            self.render_method = self.render_game_completed
            self.process_events_method = self.process_game_completed_events

        else:
            self.init_start_level()
        
        


    def start_level(self):
        # switch methods to game methods
        self.game_mode = MODE_GAME_PLAYING
        self.animation_method = self.animation
        self.render_method = self.render_game
        self.process_events_method = self.process_game_events

    def update_time_remaining(self):
        pass
        
# split pipes tiles
class Spritesheet:
    def __init__(self, filename):
        self.sheet = pygame.image.load(os.path.join('data',filename)).convert()
    
    def imgat(self, rect, colorkey = None):
        rect = Rect(rect)
        image = pygame.Surface(rect.size).convert()
        image.blit(self.sheet, (0, 0), rect)
        if colorkey is not None:
            if colorkey is -1:
                colorkey = image.get_at((0, 0))
            image.set_colorkey(colorkey, RLEACCEL)
        return image
    
    def imgsat(self, rects, colorkey = None):
        imgs = []
        for rect in rects:
            imgs.append(self.imgat(rect, colorkey))
        return imgs
    
